﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace LocalYuuPic
{
    public class ImagePreviewForm : Form
    {
        private Label infoLabel;
        private PictureBox previewBox;
        private const int ContentPadding = 10;

        public ImagePreviewForm(Form owner)
        {
            // フォームの基本設定
            this.ShowInTaskbar = false;
            this.FormBorderStyle = FormBorderStyle.None;
            this.StartPosition = FormStartPosition.Manual;
            this.Font = SystemFonts.MessageBoxFont;
            this.Owner = owner;

            // コントロールの初期化
            infoLabel = new Label
            {
                AutoSize = true,
                Location = new Point(ContentPadding, ContentPadding),
                BackColor = SystemColors.Info
            };
            this.Controls.Add(infoLabel);

            previewBox = new PictureBox
            {
                SizeMode = PictureBoxSizeMode.Zoom,
                BackColor = Color.White,
                BorderStyle = BorderStyle.FixedSingle
            };
            this.Controls.Add(previewBox);
        }

        public void ApplyTheme(bool isDark)
        {
            DarkModeUtility.ApplyDarkMode(this,isDark);
            if (isDark)
            {
                BackColor = Color.FromArgb(60, 60, 0);
                infoLabel.BackColor = BackColor;
            }
            else
            {
                BackColor = SystemColors.Info;
                infoLabel.BackColor = BackColor;
            }
        }

        protected override bool ShowWithoutActivation
        {
            get { return true; }
        }

        public void ShowPreview(string text, Image previewImage, Point location)
        {
            // ラベルのテキストを設定
            infoLabel.Text = text;

            // プレビュー画像を設定
            previewBox.Image?.Dispose();
            previewBox.Image = previewImage;

            // プレビュー画像のサイズとレイアウトを設定
            previewBox.Size = new Size(previewImage.Width, previewImage.Height);
            previewBox.Location = new Point(ContentPadding, infoLabel.Bottom + ContentPadding);

            // フォームのサイズを計算
            this.Size = new Size(
                Math.Max(infoLabel.Width, previewBox.Width) + (ContentPadding * 2),
                infoLabel.Height + previewBox.Height + (ContentPadding * 3)
            );

            // フォームの位置を設定（画面からはみ出さないように調整）
            Point adjustedLocation = AdjustLocationToScreen(location);
            this.Location = adjustedLocation;

            // フォームを表示
            if (!this.Visible)
            {
                this.Show();
            }
        }

        private Point AdjustLocationToScreen(Point desiredLocation)
        {
            // カーソルの右下に表示
            Point location = new Point(desiredLocation.X + 20, desiredLocation.Y + 20);

            // スクリーンの端を考慮
            Rectangle screenBounds = Screen.FromPoint(location).WorkingArea;
            if (location.X + this.Width > screenBounds.Right)
            {
                location.X = desiredLocation.X - this.Width - 10;
            }
            if (location.Y + this.Height > screenBounds.Bottom)
            {
                location.Y = desiredLocation.Y - this.Height - 10;
            }

            return location;
        }

        protected override CreateParams CreateParams
        {
            get
            {
                CreateParams cp = base.CreateParams;
                cp.ClassStyle |= 0x00020000; // CS_DROPSHADOW
                return cp;
            }
        }
    }
}
