﻿using System;
using System.IO;

namespace LocalYuuPic
{
    public class SizeCalculatingStream : Stream
    {
        private long _size = 0;
        private long _position = 0;

        public override bool CanRead => true;
        public override bool CanSeek => true;
        public override bool CanWrite => true;
        public override long Length => _size;
        public override long Position
        {
            get => _position;
            set => Seek(value, SeekOrigin.Begin);
        }

        public override void Write(byte[] buffer, int offset, int count)
        {
            _position += count;
            if (_position > _size)
            {
                _size = _position;
            }
        }

        public override void WriteByte(byte value)
        {
            _position++;
            if (_position > _size)
            {
                _size = _position;
            }
        }

        public override int Read(byte[] buffer, int offset, int count)
        {
            long availableBytes = _size - _position;
            if (availableBytes <= 0) return 0;

            int bytesToRead = (int)Math.Min(count, availableBytes);
            _position += bytesToRead;
            return bytesToRead;
        }

        public override long Seek(long offset, SeekOrigin origin)
        {
            switch (origin)
            {
                case SeekOrigin.Begin:
                    _position = offset;
                    break;
                case SeekOrigin.Current:
                    _position += offset;
                    break;
                case SeekOrigin.End:
                    _position = _size + offset;
                    break;
            }

            _position = Math.Max(0, Math.Min(_position, _size));
            return _position;
        }

        public override void SetLength(long value)
        {
            _size = Math.Max(0, value);
            _position = Math.Min(_position, _size);
        }

        public override void Flush() { }
    }
}
